"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateQueries = exports.getRootQueryNames = exports.getTopicsFromDocument = void 0;
const core_1 = require("@graphql-inspector/core");
const graphql_1 = require("graphql");
const getTopicsFromDocument = (document) => {
    const queries = document.definitions.filter((a) => a.kind === "OperationDefinition");
    const topicNames = queries.map((a) => {
        if (a.operation !== "query") {
            throw new Error(`Event query must be of query type: ${(0, graphql_1.print)(a)}`);
        }
        // console.log(JSON.stringify(a, null, 2));
        if (a.selectionSet.selections.length > 1) {
            throw new Error(`You must specify unique event queries: ${JSON.stringify(a.selectionSet.selections)}`);
        }
        const selection = a.selectionSet.selections[0];
        if (selection.kind !== "Field") {
            throw new Error(`Invalid query ${(0, graphql_1.print)(a)}`);
        }
        return selection.name.value;
    });
    if (Array.from(new Set(topicNames)).length < topicNames.length) {
        throw new Error("You must specify unique events");
    }
    return queries.reduce((acc, cur, curIndex) => {
        return Object.assign(Object.assign({}, acc), { [topicNames[curIndex]]: cur });
    }, {});
};
exports.getTopicsFromDocument = getTopicsFromDocument;
const validateObject = (type) => {
    if (!(0, graphql_1.isObjectType)(type)) {
        throw new Error("Return type must be an object");
    }
    // const fieldNames = Object.keys(type.getFields());
    // if (fieldNames.indexOf("id") === -1) {
    //   throw new Error("You must specify an id field for event payload");
    // }
    // if (fieldNames.indexOf("timestamp") === -1) {
    //   throw new Error(
    //     "You must specify an timestamp field for event payload"
    //   );
    // }
    // Object.values(type.getFields()).forEach((a) => {
    //   if (a.name === "id" && a.type.inspect() !== "UUID!") {
    //     throw new Error("id field must be of type UUID!");
    //   }
    //   if (a.name === "timestamp" && a.type.inspect() !== "DateTime!") {
    //     throw new Error("timestamp field must be of type DateTime!");
    //   }
    // });
};
const getRootQueryNames = (schema) => {
    const queryType = schema.getQueryType();
    if (!queryType) {
        throw new Error("Root query not found");
    }
    const queryFields = queryType.getFields();
    // console.log(queryFields);
    if (!Object.values(queryFields).every((a) => a.type.inspect().endsWith("!"))) {
        throw new Error("You must specify all queries with non null response type");
    }
    const returnTypes = Object.values(queryFields).map((a) => a.type.toString());
    for (const returnType of returnTypes) {
        const type = schema.getType(returnType.split("!")[0]);
        if (!(0, graphql_1.isObjectType)(type) && !(0, graphql_1.isUnionType)(type)) {
            throw new Error("You must return objects for events");
        }
        if ((0, graphql_1.isObjectType)(type)) {
            validateObject(type);
        }
        else {
            type.getTypes().forEach((t) => validateObject(t));
        }
        // console.log(JSON.stringify(type?.astNode, null, 2));
    }
    return Object.keys(queryFields);
};
exports.getRootQueryNames = getRootQueryNames;
const validateQueries = (schema, queries) => {
    const result = (0, core_1.validate)(schema, queries.map((a) => new graphql_1.Source((0, graphql_1.print)(a))));
    if (result.length > 0 && result.find((a) => a.errors.length > 0)) {
        throw new Error(`Invalid queries found: ${JSON.stringify(result, null, 2)}`);
    }
    if (result.length > 0 &&
        result.find((a) => a.deprecated.length > 0)) {
        console.warn(`Deprecated fields found: ${JSON.stringify(result.map((a) => a.deprecated), null, 2)}`);
    }
};
exports.validateQueries = validateQueries;
